#  ----------------------------------------------------------------------------
#  <copyright company="Aspose" file="ClientMessageMoveRequest.rb">
#    Copyright (c) 2018-2020 Aspose Pty Ltd. All rights reserved.
#  </copyright>
#  <summary>
#    Permission is hereby granted, free of charge, to any person obtaining a
#   copy  of this software and associated documentation files (the "Software"),
#   to deal  in the Software without restriction, including without limitation
#   the rights  to use, copy, modify, merge, publish, distribute, sublicense,
#   and/or sell  copies of the Software, and to permit persons to whom the
#   Software is  furnished to do so, subject to the following conditions:
#
#   The above copyright notice and this permission notice shall be included in
#   all  copies or substantial portions of the Software.
#
#   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#   FROM,  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#   DEALINGS IN THE SOFTWARE.
#  </summary>
#  ----------------------------------------------------------------------------

require 'date'

module AsposeEmailCloud
  # Email client move message request.             
  class ClientMessageMoveRequest < ClientMessageBaseRequest
    # Email client account configuration location on storage.             
    # @return [StorageFileLocation]
    attr_accessor :account_location
    # Message identifier.             
    # @return [String]
    attr_accessor :message_id
    # Folder to move message from.             
    # @return [String]
    attr_accessor :source_folder
    # Folder to move message to.             
    # @return [String]
    attr_accessor :destination_folder
    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :'account_location' => :'accountLocation',
        :'message_id' => :'messageId',
        :'source_folder' => :'sourceFolder',
        :'destination_folder' => :'destinationFolder'
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        :'account_location' => :'StorageFileLocation',
        :'message_id' => :'String',
        :'source_folder' => :'String',
        :'destination_folder' => :'String'
      }
    end

    # Initializes the object
    # @param [StorageFileLocation] account_location Email client account configuration location on storage.             
    # @param [String] message_id Message identifier.             
    # @param [String] source_folder Folder to move message from.             
    # @param [String] destination_folder Folder to move message to.             
    def initialize(
      account_location: nil,
      message_id: nil,
      source_folder: nil,
      destination_folder: nil)
      self.account_location = account_location if account_location
      self.message_id = message_id if message_id
      self.source_folder = source_folder if source_folder
      self.destination_folder = destination_folder if destination_folder
    end

    # Show invalid properties with the reasons. Usually used together with valid?
    # @return Array for valid properties with the reasons
    def list_invalid_properties
      invalid_properties = Array.new
      if @account_location.nil?
        invalid_properties.push('invalid value for "account_location", account_location cannot be nil.')
      end

      if @message_id.nil?
        invalid_properties.push('invalid value for "message_id", message_id cannot be nil.')
      end

      if @message_id.to_s.length < 1
        invalid_properties.push('invalid value for "message_id", the character length must be great than or equal to 1.')
      end

      if @destination_folder.nil?
        invalid_properties.push('invalid value for "destination_folder", destination_folder cannot be nil.')
      end

      if @destination_folder.to_s.length < 1
        invalid_properties.push('invalid value for "destination_folder", the character length must be great than or equal to 1.')
      end

      invalid_properties
    end

    # Check to see if the all the properties in the model are valid
    # @return true if the model is valid
    def valid?
      return false if @account_location.nil?
      return false if @message_id.nil?
      return false if @message_id.to_s.length < 1
      return false if @destination_folder.nil?
      return false if @destination_folder.to_s.length < 1
      true
    end

    # Custom attribute writer method with validation
    # @param [Object] message_id Value to be assigned
    def message_id=(message_id)
      if message_id.nil?
        fail ArgumentError, 'message_id cannot be nil'
      end

      if message_id.to_s.length < 1
        fail ArgumentError, 'invalid value for "message_id", the character length must be great than or equal to 1.'
      end

      @message_id = message_id
    end

    # Custom attribute writer method with validation
    # @param [Object] destination_folder Value to be assigned
    def destination_folder=(destination_folder)
      if destination_folder.nil?
        fail ArgumentError, 'destination_folder cannot be nil'
      end

      if destination_folder.to_s.length < 1
        fail ArgumentError, 'invalid value for "destination_folder", the character length must be great than or equal to 1.'
      end

      @destination_folder = destination_folder
    end

    # Checks equality by comparing each attribute.
    # @param [Object] Object to be compared
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          account_location == o.account_location &&
          message_id == o.message_id &&
          source_folder == o.source_folder &&
          destination_folder == o.destination_folder
    end

    # @see the `==` method
    # @param [Object] Object to be compared
    def eql?(o)
      self == o
    end

    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [account_location, message_id, source_folder, destination_folder].hash
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.swagger_types.each_pair do |key, type|
        attribute_key = self.class.attribute_map[key]
        attribute_key = (attribute_key[0, 1].downcase + attribute_key[1..-1]).to_sym
        if type =~ /\AArray<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[attribute_key].is_a?(Array)
            self.send("#{key}=", attributes[attribute_key].map { |v| _deserialize($1, v) })
          end
        elsif !attributes[attribute_key].nil?
          self.send("#{key}=", _deserialize(type, attributes[attribute_key]))
        end # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end

    # Deserializes the data based on type
    # @param string type Data type
    # @param string value Value to be deserialized
    # @return [Object] Deserialized data
    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when :Object
        # generic object (usually a Hash), return directly
        value
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+?), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        sub_type = value[:type] || value[:discriminator] || type
        if AsposeEmailCloud.const_defined?(sub_type)
          type = sub_type
        end
        temp_model = AsposeEmailCloud.const_get(type).new
        temp_model.build_from_hash(value)
      end
    end

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_hash (backward compatibility)
    # @return [Hash] Returns the object in the form of hash
    def to_body
      to_hash
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end

  end
end
